/*
 * Copyright (c) 2004-2013 Zementis, Inc. All rights reserved.
 */
package com.zementis.adapa.sample.loghandlers;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import com.zementis.adapa.extensions.ModelLogHandler;

/**
 *
 */
public class FileLogHandler implements ModelLogHandler {

    private final File recordLog;

    private final File invalidMissingValuesLog;

    private final Map<String, Integer> fieldMap;

    private final List<String> fieldNames;

    private final long[] invalidCounterArray;

    private final long[] missingCounterArray;

    public FileLogHandler(String modelName, Map<String, Class<?>> modelFields, File outputLocation) {
        this.recordLog = new File(outputLocation, modelName + "_record.csv");
        this.invalidMissingValuesLog = new File(outputLocation, modelName + "_invalidMissingValues.csv");

        this.fieldMap = new LinkedHashMap<String, Integer>(modelFields.size());
        this.invalidCounterArray = new long[modelFields.size()];
        this.missingCounterArray = new long[modelFields.size()];

        this.fieldNames = new ArrayList<String>(modelFields.keySet());
        for (Entry<String, Class<?>> entry : modelFields.entrySet()) {
            fieldNames.add(entry.getKey());
        }

        // initialize recordLog file with the header
        try {
            FileWriter writer = new FileWriter(recordLog);
            StringBuilder sb = new StringBuilder(fieldNames.get(0));
            for (int i = 1; i < fieldNames.size(); ++i) {
                sb.append(",").append(fieldNames.get(i));
            }
            sb.append(System.getProperty("line.separator"));
            writer.write(sb.toString());
            writer.flush();
            writer.close();
        } catch (IOException ioe) {
            throw new RuntimeException(ioe);
        }
    }

    @Override
    public void close() {
        flush();
    }

    @Override
    public void flush() {
        try {
            FileWriter writer = new FileWriter(invalidMissingValuesLog);
            writer.write("FieldName,InvalidCount,MissingCount");
            writer.write(System.getProperty("line.separator"));
            for (Entry<String, Integer> entry : fieldMap.entrySet()) {
                StringBuilder sb = new StringBuilder(entry.getKey());
                sb.append(',').append(invalidCounterArray[entry.getValue()]);
                sb.append(',').append(missingCounterArray[entry.getValue()]);
                sb.append(System.getProperty("line.separator"));
                writer.write(sb.toString());
            }
            writer.flush();
            writer.close();
        } catch (IOException ioe) {
            throw new RuntimeException(ioe);
        }
    }

    @Override
    public void logInvalidValue(String fieldName, Object value) {
        Integer fieldIndex = getFieldIndex(fieldName);
        invalidCounterArray[fieldIndex]++;
    }

    @Override
    public void logMissingValue(String fieldName, Object value) {
        Integer fieldIndex = getFieldIndex(fieldName);
        missingCounterArray[fieldIndex]++;
    }

    @Override
    public void logRecord(Map<String, Object> record) {
        try {
            FileWriter writer = new FileWriter(recordLog, true);
            StringBuilder sb = new StringBuilder(record.get(fieldNames.get(0)).toString());
            for (int i = 1; i < fieldNames.size(); ++i) {
                sb.append(",").append(
                    record.get(fieldNames.get(i)) != null ? record.get(fieldNames.get(i)).toString() : "");
            }
            sb.append(System.getProperty("line.separator"));
            writer.write(sb.toString());
            writer.flush();
            writer.close();
        } catch (IOException ioe) {
            throw new RuntimeException(ioe);
        }
    }

    /**
     * @param fieldName
     * @return
     */
    private Integer getFieldIndex(String fieldName) {
        Integer fieldIndex = fieldMap.get(fieldName);
        if (fieldIndex == null) {
            fieldIndex = fieldMap.size();
            fieldMap.put(fieldName, fieldIndex);
        }
        return fieldIndex;
    }

}
